// Send a message using sendmail

#include "sendmail.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>

int system_sendmail(const char *message) {
	int pipefd[2];
	if (pipe(pipefd))
		return -1;

	pid_t pid = fork();
	switch (pid) {
	case -1:
		close(pipefd[0]);
		close(pipefd[1]);
		return -1;
	case 0:
		if (close(pipefd[1]) || close(STDIN_FILENO)
			|| dup2(pipefd[0], STDIN_FILENO) == -1
			|| !freopen("/dev/null", "w", stdout)
			|| !freopen("/dev/null", "w", stderr))
			exit(EXIT_FAILURE);
		static const char SENDMAIL[] = "/usr/sbin/sendmail";
		execl(SENDMAIL, SENDMAIL, "-bm", "-ti", NULL);
		exit(EXIT_FAILURE);
	}

	FILE *mailpipe = NULL;
	_Bool ok = 0;
	do {
		if (close(pipefd[0]))
			break;
		mailpipe = fdopen(pipefd[1], "w");
		if (!mailpipe)
			break;
		if (fputs(message, mailpipe) == EOF)
			break;
		if (fflush(mailpipe))
			break;
		ok = 1;
	} while (0);

	if (mailpipe) {
		if (fclose(mailpipe))
			ok = 0;
	}
	else
		close(pipefd[1]);

	siginfo_t info;
	if (waitid(P_PID, pid, &info, WEXITED))
		return -1;
	if (!(ok && info.si_code == CLD_EXITED
		&& info.si_status == EXIT_SUCCESS))
		return -1;
	return 0;
}
